@@ -64,6 +64,7 @@
 /* Signals */
 enum {
 	HIGHLIGHT_UPDATED,
+	SOURCE_MARK_UPDATED,
 	LAST_SIGNAL
 };
 
@@ -88,6 +89,8 @@ struct _GtkSourceBufferPrivate
 	GtkTextMark           *bracket_mark;
 	guint                  bracket_found:1;
 
+	GArray                *source_marks;
+
 	GtkSourceLanguage     *language;
 
 	GtkSourceEngine       *highlight_engine;
@@ -95,10 +98,9 @@ struct _GtkSourceBufferPrivate
 
 	GtkSourceUndoManager  *undo_manager;
 };
-
+	
 G_DEFINE_TYPE (GtkSourceBuffer, gtk_source_buffer, GTK_TYPE_TEXT_BUFFER)
 
-
 static guint 	 buffer_signals[LAST_SIGNAL];
 
 static void 	 gtk_source_buffer_finalize		(GObject                 *object);
@@ -111,19 +113,12 @@ static void      gtk_source_buffer_get_property         (GObject
 							 guint                    prop_id,
 							 GValue                  *value,
 							 GParamSpec              *pspec);
-
 static void 	 gtk_source_buffer_can_undo_handler 	(GtkSourceUndoManager    *um,
 							 gboolean                 can_undo,
 							 GtkSourceBuffer         *buffer);
 static void 	 gtk_source_buffer_can_redo_handler	(GtkSourceUndoManager    *um,
 							 gboolean                 can_redo,
 							 GtkSourceBuffer         *buffer);
-
-static void 	 gtk_source_buffer_move_cursor		(GtkTextBuffer           *buffer,
-							 const GtkTextIter       *iter,
-							 GtkTextMark             *mark,
-							 gpointer                 data);
-
 static void 	 gtk_source_buffer_real_insert_text 	(GtkTextBuffer           *buffer,
 							 GtkTextIter             *iter,
 							 const gchar             *text,
@@ -131,7 +126,11 @@ static void 	 gtk_source_buffer_real_insert_text 	(GtkTextBuffer           *buff
 static void 	 gtk_source_buffer_real_delete_range 	(GtkTextBuffer           *buffer,
 							 GtkTextIter             *iter,
 							 GtkTextIter             *end);
-
+static void 	 gtk_source_buffer_real_mark_set	(GtkTextBuffer		 *buffer,
+							 const GtkTextIter	 *location,
+							 GtkTextMark		 *mark);
+static void 	 gtk_source_buffer_real_mark_deleted	(GtkTextBuffer		 *buffer,
+							 GtkTextMark		 *mark);
 static gboolean	 gtk_source_buffer_find_bracket_match_with_limit (GtkTextIter    *orig,
 								  gint            max_chars);
 
@@ -152,8 +151,11 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 
 	/* Do not set these signals handlers directly on the parent_class since
 	 * that will cause problems (a loop). */
+	tb_class->delete_range  = gtk_source_buffer_real_delete_range;
 	tb_class->insert_text 	= gtk_source_buffer_real_insert_text;
-	tb_class->delete_range 	= gtk_source_buffer_real_delete_range;
+	
+	tb_class->mark_set	= gtk_source_buffer_real_mark_set;
+	tb_class->mark_deleted	= gtk_source_buffer_real_mark_deleted;
 
 	/**
 	 * GtkSourceBuffer:highlight-syntax:
@@ -252,6 +254,22 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 			   _gtksourceview_marshal_VOID__BOXED_BOXED,
 			   G_TYPE_NONE,
 			   2, param_types);
+	/**
+	 * GtkSourceBuffer::source-mark-updated
+	 * @buffer: the buffer that received the signal
+	 *
+	 * The ::source_mark_updated signal is emitted each time 
+	 * a mark is added to, moved or removed from the @buffer.
+	 **/
+	buffer_signals[SOURCE_MARK_UPDATED] =
+	    g_signal_new ("source_mark_updated",
+			   G_OBJECT_CLASS_TYPE (object_class),
+			   G_SIGNAL_RUN_LAST,
+			   0,
+			   NULL, NULL,
+			   g_cclosure_marshal_VOID__OBJECT,
+			   G_TYPE_NONE,
+			   1, GTK_TYPE_TEXT_MARK);
 
 	g_type_class_add_private (object_class, sizeof(GtkSourceBufferPrivate));
 }
@@ -273,14 +291,12 @@ gtk_source_buffer_init (GtkSourceBuffer *buffer)
 	priv->bracket_mark = NULL;
 	priv->bracket_found = FALSE;
 
+	priv->source_marks = g_array_new (FALSE, FALSE, sizeof (GtkSourceMark *));
+
 	priv->style_scheme = _gtk_source_style_scheme_get_default ();
 	if (priv->style_scheme != NULL)
 		g_object_ref (priv->style_scheme);
 
-	g_signal_connect (buffer,
-			  "mark_set",
-			  G_CALLBACK (gtk_source_buffer_move_cursor),
-			  NULL);
 	g_signal_connect (priv->undo_manager,
 			  "can_undo",
 			  G_CALLBACK (gtk_source_buffer_can_undo_handler),
@@ -302,6 +318,9 @@ gtk_source_buffer_finalize (GObject *object)
 	buffer = GTK_SOURCE_BUFFER (object);
 	g_return_if_fail (buffer->priv != NULL);
 
+	if (buffer->priv->source_marks)
+		g_array_free (buffer->priv->source_marks, TRUE);
+
 	G_OBJECT_CLASS (gtk_source_buffer_parent_class)->finalize (object);
 }
 
@@ -533,8 +552,7 @@ get_bracket_match_tag (GtkSourceBuffer *buffer)
 static void
 gtk_source_buffer_move_cursor (GtkTextBuffer     *buffer,
 			       const GtkTextIter *iter,
-			       GtkTextMark       *mark,
-			       gpointer           data)
+			       GtkTextMark       *mark)
 {
 	GtkTextIter iter1, iter2;
 
@@ -612,12 +630,11 @@ gtk_source_buffer_real_insert_text (GtkTextBuffer *buffer,
 	 * default signal handler revalidates it to point to the end of the
 	 * inserted text
 	 */
-	GTK_TEXT_BUFFER_CLASS(gtk_source_buffer_parent_class)->insert_text (buffer, iter, text, len);
+	GTK_TEXT_BUFFER_CLASS (gtk_source_buffer_parent_class)->insert_text (buffer, iter, text, len);
 
 	gtk_source_buffer_move_cursor (buffer,
 				       iter,
-				       gtk_text_buffer_get_insert (buffer),
-				       NULL);
+				       gtk_text_buffer_get_insert (buffer));
 
 	end_offset = gtk_text_iter_get_offset (iter);
 
@@ -647,21 +664,16 @@ gtk_source_buffer_real_delete_range (GtkTextBuffer *buffer,
 	offset = gtk_text_iter_get_offset (start);
 	length = gtk_text_iter_get_offset (end) - offset;
 
-	GTK_TEXT_BUFFER_CLASS(gtk_source_buffer_parent_class)->delete_range (buffer, start, end);
+	GTK_TEXT_BUFFER_CLASS (gtk_source_buffer_parent_class)->delete_range (buffer, start, end);
 
 	mark = gtk_text_buffer_get_insert (buffer);
 	gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
-
-	gtk_source_buffer_move_cursor (buffer,
-				       &iter,
-				       mark,
-				       NULL);
+	gtk_source_buffer_move_cursor (buffer, &iter, mark);
 
 	/* emit text deleted for engines */
 	if (source_buffer->priv->highlight_engine != NULL)
 		_gtk_source_engine_text_deleted (source_buffer->priv->highlight_engine,
-						 offset,
-						 length);
+						 offset, length);
 }
 
 /* FIXME: this can't be here, but it's now needed for bracket matching */
@@ -1011,7 +1023,7 @@ gtk_source_buffer_get_highlight_matching_brackets (GtkSourceBuffer *buffer)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), FALSE);
 
-	return buffer->priv->highlight_brackets;
+	return (buffer->priv->highlight_brackets != FALSE);
 }
 
 /**
@@ -1055,7 +1067,7 @@ gtk_source_buffer_get_highlight_syntax (GtkSourceBuffer *buffer)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), FALSE);
 
-	return buffer->priv->highlight_syntax;
+	return (buffer->priv->highlight_syntax != FALSE);
 }
 
 /**
@@ -1080,8 +1092,8 @@ gtk_source_buffer_set_highlight_syntax (GtkSourceBuffer *buffer,
 
 	if (buffer->priv->highlight_syntax != highlight)
 	{
-	buffer->priv->highlight_syntax = highlight;
-	g_object_notify (G_OBJECT (buffer), "highlight-syntax");
+		buffer->priv->highlight_syntax = highlight;
+		g_object_notify (G_OBJECT (buffer), "highlight-syntax");
 	}
 }
 
@@ -1091,7 +1103,7 @@ gtk_source_buffer_set_highlight_syntax (GtkSourceBuffer *buffer,
  * @language: a #GtkSourceLanguage to set, or %NULL.
  *
  * Associate a #GtkSourceLanguage with the source buffer. If @language is 
- * not-%NULL and syntax highlighting is enabled (see gtk_source_buffer_set_highlight()),
+ * not-%NULL and syntax highlighting is enabled (see gtk_source_buffer_set_highlight_syntax()),
  * the syntax patterns defined in @language will be used to highlight the text
  * contained in the buffer. If @language is %NULL, the text contained in the 
  * buffer is not highlighted.
@@ -1256,3 +1268,552 @@ gtk_source_buffer_get_style_scheme (GtkSourceBuffer *buffer)
 	return buffer->priv->style_scheme;
 }
 
+/* Source Marks functionality */
+
+/* At the moment this is pretty dumb (O(N)), if it is a performance
+ * problem we should change data struct.
+ * Since it's used from mark_set when the mark was moved, we cannot bsearch.
+ * Returns TRUE if the mark was found and removed */
+static gboolean
+source_mark_remove (GtkSourceBuffer *buffer, GtkSourceMark *mark)
+{
+	gint i;
+
+	for (i = 0; i < buffer->priv->source_marks->len; ++i)
+	{
+		GtkSourceMark *m;
+
+		m = g_array_index (buffer->priv->source_marks, GtkSourceMark *, i);
+		if (mark == m)
+		{
+			g_array_remove_index (buffer->priv->source_marks, i);
+			g_object_unref (m);
+
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+/* Performs a binary search among the source marks in @buffer for the
+ * position of the @iter.  Returns the nearest matching mark (its
+ * index in the marks array) and optionally the value of the
+ * comparision between the given iter and the iter at the returned mark.
+ *
+ * Return value: an index in the source marks array or -1 if the array is
+ * empty.
+ */
+static gint
+source_mark_bsearch (GtkSourceBuffer *buffer, GtkTextIter *iter, gint *last_cmp)
+{
+	GtkTextIter check_iter;
+	GtkSourceMark **check, **p;
+	GArray *marks = buffer->priv->source_marks;
+	gint n_marks = marks->len;
+	gint cmp, i;
+
+	if (n_marks == 0)
+		return -1;
+
+	check = p = &g_array_index (marks, GtkSourceMark *, 0);
+	p--;
+	cmp = 0;
+	while (n_marks)
+	{
+		i = (n_marks + 1) >> 1;
+		check = p + i;
+		gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+						  &check_iter,
+						  GTK_TEXT_MARK (*check));
+		cmp = gtk_text_iter_compare (iter, &check_iter);
+		if (cmp > 0)
+		{
+			n_marks -= i;
+			p = check;
+		}
+		else if (cmp < 0)
+			n_marks = i - 1;
+		else /* if (cmp == 0) */
+			break;
+	}
+
+	i = check - &g_array_index (marks, GtkSourceMark *, 0);
+	if (last_cmp)
+		*last_cmp = cmp;
+
+	return i;
+}
+
+static void
+source_mark_insert (GtkSourceBuffer *buffer, GtkSourceMark *mark)
+{
+	GtkTextIter iter;
+	gint index, cmp;
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+					  &iter,
+					  GTK_TEXT_MARK (mark));
+
+	index = source_mark_bsearch (buffer, &iter, &cmp);
+	if (index >= 0)
+	{
+		/* if the mark we found is at same iter or before
+		 * put our mark after that */
+		if (cmp >= 0)
+			index++;
+	}
+	else
+	{
+		index = 0;
+	}
+
+	g_object_ref (mark);
+	g_array_insert_val (buffer->priv->source_marks, index, mark);
+}
+
+static void
+gtk_source_buffer_real_mark_set	(GtkTextBuffer     *buffer,
+				 const GtkTextIter *location,
+				 GtkTextMark       *mark)
+{
+	if (GTK_IS_SOURCE_MARK (mark))
+	{
+		/* for now we simply remove and reinsert at
+		 * the right place every time */
+		source_mark_remove (GTK_SOURCE_BUFFER (buffer),
+				    GTK_SOURCE_MARK (mark));
+		source_mark_insert (GTK_SOURCE_BUFFER (buffer),
+				    GTK_SOURCE_MARK (mark));
+
+		g_signal_emit_by_name (buffer, "source_mark_updated", mark);
+	}
+
+	/* if the mark is the insert mark, update bracket matching */
+	else if (mark == gtk_text_buffer_get_insert (buffer))
+	{
+		gtk_source_buffer_move_cursor (buffer, location, mark);
+	}
+
+	GTK_TEXT_BUFFER_CLASS (gtk_source_buffer_parent_class)->mark_set (buffer, location, mark);
+}
+
+static void 	 
+gtk_source_buffer_real_mark_deleted (GtkTextBuffer *buffer,
+				     GtkTextMark *mark)
+{
+	if (GTK_IS_SOURCE_MARK (mark))
+	{
+		source_mark_remove (GTK_SOURCE_BUFFER (buffer),
+				    GTK_SOURCE_MARK (mark));
+
+		g_signal_emit_by_name (buffer, "source_mark_updated", mark);
+	}
+
+	if (GTK_TEXT_BUFFER_CLASS (gtk_source_buffer_parent_class)->mark_deleted != NULL)
+		GTK_TEXT_BUFFER_CLASS (gtk_source_buffer_parent_class)->mark_deleted (buffer, mark);
+}
+
+/**
+ * gtk_source_buffer_create_mark:
+ * @buffer: a #GtkSourceBuffer.
+ * @name: the name of the mark, or %NULL.
+ * @category: a string defining the mark category.
+ * @where: location to place the mark.
+ *
+ * Creates a source mark in the @buffer of category @category.  A source mark is
+ * a #GtkTextMark but organised into categories. Depending on the category
+ * a pixbuf can be specified that will be displayed along the line of the mark.
+ *
+ * Like a #GtkTextMark, a #GtkSourceMark can be anonymous if the
+ * passed @name is %NULL.  Also, the buffer owns the marks so you
+ * shouldn't unreference it.
+ *
+ * Marks always have left gravity and are moved to the beginning of
+ * the line when the user deletes the line they were in.
+ *
+ * Typical uses for a source mark are bookmarks, breakpoints, current
+ * executing instruction indication in a source file, etc..
+ *
+ * Return value: a new #GtkSourceMark, owned by the buffer.
+ *
+ * Since: 2.2
+ **/
+GtkSourceMark *
+gtk_source_buffer_create_mark (GtkSourceBuffer   *buffer,
+			       const gchar       *name,
+			       const gchar       *category,
+			       const GtkTextIter *where)
+{
+	GtkSourceMark *mark;
+
+	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), NULL);
+	g_return_val_if_fail (category != NULL, NULL);
+	g_return_val_if_fail (where != NULL, NULL);
+
+	mark = gtk_source_mark_new (name, category);
+	gtk_text_buffer_add_mark (GTK_TEXT_BUFFER (buffer),
+				  GTK_TEXT_MARK (mark),
+				  where);
+
+	return mark;
+}
+
+GtkSourceMark *
+_gtk_source_buffer_mark_next (GtkSourceBuffer *buffer,
+			      GtkSourceMark   *mark,
+			      const gchar     *category)
+{
+	GtkTextIter iter;
+	gint index, cmp;
+
+	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), NULL);
+
+	/* TODO: we could speed this up by caching the current
+	 * position in the mark and invalidating the cache when
+	 * the marks array changes. For now we always lookup. */
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+					  &iter,
+					  GTK_TEXT_MARK (mark));
+
+	index = source_mark_bsearch (buffer, &iter, &cmp);
+
+	/* the array should already contain @mark */
+	g_return_val_if_fail (index >= 0, NULL);
+	g_return_val_if_fail (cmp == 0, NULL);
+
+	/* move up to our mark among the ones at this position */
+	while (mark != g_array_index (buffer->priv->source_marks, GtkSourceMark *, index))
+	{
+		++index;
+	}
+
+	while (++index < buffer->priv->source_marks->len)
+	{
+		GtkSourceMark *ret;
+
+		ret = g_array_index (buffer->priv->source_marks, GtkSourceMark *, index);
+		if (category == NULL ||
+		    0 == strcmp (category, gtk_source_mark_get_category (ret)))
+		{
+			return ret;
+		}
+	}
+
+	return NULL;
+}
+
+GtkSourceMark *
+_gtk_source_buffer_mark_prev (GtkSourceBuffer *buffer,
+			      GtkSourceMark   *mark,
+			      const gchar     *category)
+{
+	GtkTextIter iter;
+	gint index, cmp;
+
+	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), NULL);
+
+	/* TODO: we could speed this up by caching the current
+	 * position in the mark and invalidating the cache when
+	 * the marks array changes. For now we always lookup. */
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+					  &iter,
+					  GTK_TEXT_MARK (mark));
+
+	index = source_mark_bsearch (buffer, &iter, &cmp);
+
+	/* the array should already contain @mark */
+	g_return_val_if_fail (index >= 0, NULL);
+	g_return_val_if_fail (cmp == 0, NULL);
+
+	/* move up to our mark among the ones at this position */
+	while (mark != g_array_index (buffer->priv->source_marks, GtkSourceMark *, index))
+	{
+		++index;
+	}
+
+	while (--index >= 0)
+	{
+		GtkSourceMark *ret;
+
+		ret = g_array_index (buffer->priv->source_marks, GtkSourceMark *, index);
+		if (category == NULL ||
+		    0 == strcmp (category, gtk_source_mark_get_category (ret)))
+		{
+			return ret;
+		}
+	}
+
+	return NULL;
+}
+
+/**
+ * gtk_source_buffer_forward_iter_to_mark:
+ * @buffer: a #GtkSourceBuffer.
+ * @iter: an iterator.
+ * @category: category to search for or %NULL
+ *
+ * Moves @iter to the position of the next #GtkSourceMark of the given
+ * @category. Returns #TRUE if @iter was moved. If @category is NULL, the
+ * next source mark can be of any category.
+ *
+ * Returns: whether iter moved.
+ *
+ * Since: 2.2
+ **/
+gboolean
+gtk_source_buffer_forward_iter_to_mark (GtkSourceBuffer *buffer,
+					GtkTextIter     *iter,
+					const gchar     *category)
+{
+	GtkTextIter i;
+	gint index, cmp;
+
+	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), FALSE);
+	g_return_val_if_fail (iter != NULL, FALSE);
+
+	i = *iter;
+
+	index = source_mark_bsearch (buffer, &i, &cmp);
+	if (index < 0)
+		return FALSE;
+
+	if (cmp >= 0)
+		++index;
+
+	while (index < buffer->priv->source_marks->len)
+	{
+		GtkSourceMark *mark;
+
+		mark = g_array_index (buffer->priv->source_marks, GtkSourceMark *, index);
+		if (category == NULL ||
+		    0 == strcmp (category, gtk_source_mark_get_category (mark)))
+		{
+			/* update the iter */
+			gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+							  &i, GTK_TEXT_MARK (mark));
+
+			if (gtk_text_iter_compare (&i, iter) > 0 )
+			{
+				*iter = i;
+				return TRUE;
+			}
+		}
+
+		++index;
+	}
+
+	return FALSE;
+}
+
+/**
+ * gtk_source_buffer_backward_iter_to_mark:
+ * @buffer: a #GtkSourceBuffer.
+ * @iter: an iterator.
+ * @category: category to search for or %NULL
+ *
+ * Moves @iter to the position of the previous #GtkSourceMark of the given
+ * category. Returns #TRUE if @iter was moved. If @category is NULL, the
+ * previous source mark can be of any category.
+ *
+ * Returns: whether iter moved.
+ *
+ * Since: 2.2
+ **/
+gboolean
+gtk_source_buffer_backward_iter_to_mark (GtkSourceBuffer *buffer,
+					 GtkTextIter     *iter,
+					 const gchar     *category)
+{
+	GtkTextIter i;
+	gint index, cmp;
+
+	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), FALSE);
+	g_return_val_if_fail (iter != NULL, FALSE);
+
+	i = *iter;
+
+	index = source_mark_bsearch (buffer, &i, &cmp);
+	if (index < 0)
+		return FALSE;
+
+	if (cmp <= 0)
+		--index;
+
+	while (index >= 0)
+	{
+		GtkSourceMark *mark;
+
+		mark = g_array_index (buffer->priv->source_marks, GtkSourceMark *, index);
+		if (category == NULL ||
+		    0 == strcmp (category, gtk_source_mark_get_category (mark)))
+		{
+			gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+							  &i, GTK_TEXT_MARK (mark));
+
+			if (gtk_text_iter_compare (&i, iter) < 0 )
+			{
+				*iter = i;
+				return TRUE;
+			}
+		}
+
+		--index;
+	}
+
+	return FALSE;
+}
+
+/**
+ * gtk_source_buffer_get_marks_at_iter:
+ * @buffer: a #GtkSourceBuffer.
+ * @iter: an iterator.
+ * @category: category to search for or %NULL
+ *
+ * Returns the list of marks of the given category at @iter. If @category
+ * is %NULL it returns all marks at @iter.
+ *
+ * Returns: a newly allocated #GSList.
+ *
+ * Since: 2.2
+ **/
+GSList *
+gtk_source_buffer_get_marks_at_iter (GtkSourceBuffer *buffer,
+				     GtkTextIter     *iter,
+				     const gchar     *category)
+{
+	GSList *marks, *l, *res;
+
+	res = NULL;
+	marks = gtk_text_iter_get_marks (iter);
+
+	for (l = marks; l != NULL; l = l->next)
+	{
+		GtkSourceMark *mark;
+
+		if (!GTK_IS_SOURCE_MARK (l->data))
+			continue;
+
+		mark = GTK_SOURCE_MARK (l->data);
+		if (category == NULL ||
+		    0 == strcmp (category, gtk_source_mark_get_category (mark)))
+		{
+			res = g_slist_prepend (res, l->data);
+		}
+	}
+
+	return g_slist_reverse (res);
+}
+
+/**
+ * gtk_source_buffer_get_marks_at_line:
+ * @buffer: a #GtkSourceBuffer.
+ * @line: a line number.
+ * @category: category to search for or %NULL
+ *
+ * Returns the list of marks of the given category at @line.
+ * If @category is NULL, all marks at @line are returned.
+ *
+ * Returns: a newly allocated #GSList.
+ *
+ * Since: 2.2
+ **/
+GSList *
+gtk_source_buffer_get_marks_at_line (GtkSourceBuffer *buffer,
+				     gint             line,
+				     const gchar     *category)
+{
+	GtkTextIter iter;
+	GSList *res;
+
+ 	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), NULL);
+
+	gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (buffer),
+					  &iter, line);
+
+	res = gtk_source_buffer_get_marks_at_iter (buffer,
+						   &iter,
+						   category); 
+
+	while (gtk_source_buffer_forward_iter_to_mark (buffer,
+						       &iter,
+						       category))
+	{
+		if (gtk_text_iter_get_line (&iter) == line)
+		{
+			GSList *l;
+
+			l =  gtk_source_buffer_get_marks_at_iter (buffer,
+								  &iter,
+								  category);
+
+			res = g_slist_concat (res, l);
+		}
+		else
+		{
+			break;
+		}
+	}
+
+	return res;
+}
+
+/**
+ * gtk_source_buffer_remove_marks:
+ * @buffer: a #GtkSourceBuffer.
+ * @start: a #GtkTextIter
+ * @end: a #GtkTextIter
+ * @category: category to search for or NULL
+ *
+ * Remove all marks of @category between @start and @end from the buffer.
+ * If @category is NULL, all marks in the range will be removed.
+ *
+ * Since: 2.2
+ **/
+void
+gtk_source_buffer_remove_marks (GtkSourceBuffer *buffer,
+				GtkTextIter     *start,
+				GtkTextIter     *end,
+				const gchar     *category)
+{
+	GtkTextIter iter;
+	GSList *list;
+	GSList *l;
+
+ 	g_return_if_fail (GTK_IS_SOURCE_BUFFER (buffer));
+ 	g_return_if_fail (start != NULL);
+ 	g_return_if_fail (end != NULL);
+
+	iter = *start;
+
+	list = gtk_source_buffer_get_marks_at_iter (buffer,
+						    &iter,
+						    category); 
+
+	while (gtk_source_buffer_forward_iter_to_mark (buffer,
+						       &iter,
+						       category))
+	{
+		if (gtk_text_iter_compare (&iter, end) <= 0)
+		{
+			l =  gtk_source_buffer_get_marks_at_iter (buffer,
+								  &iter,
+								  category);
+
+			list = g_slist_concat (list, l);
+		}
+		else
+		{
+			break;
+		}
+	}
+
+	for (l = list; l != NULL; l = l->next)
+	{
+		gtk_text_buffer_delete_mark (GTK_TEXT_BUFFER (buffer),
+					     GTK_TEXT_MARK (l->data));
+	}
+
+	g_slist_free (list);
+}
+
