@@ -28,7 +28,7 @@
 
 #include <gtk/gtk.h>
 #include <gtksourceview/gtksourcelanguage.h>
-#include <gtksourceview/gtksourcemarker.h>
+#include <gtksourceview/gtksourcemark.h>
 #include <gtksourceview/gtksourcestylescheme.h>
 
 G_BEGIN_DECLS
@@ -109,39 +109,39 @@ void			 gtk_source_buffer_redo			(GtkSourceBuffer        *buffer);
 void			 gtk_source_buffer_begin_not_undoable_action (GtkSourceBuffer   *buffer);
 void			 gtk_source_buffer_end_not_undoable_action   (GtkSourceBuffer   *buffer);
 
-/* Marker methods */
-GtkSourceMarker         *gtk_source_buffer_create_marker        (GtkSourceBuffer        *buffer,
+/* Mark methods */
+GtkSourceMark		*gtk_source_buffer_create_mark		(GtkSourceBuffer        *buffer,
 								 const gchar            *name,
-								 const gchar            *type,
+								 const gchar            *category,
 								 const GtkTextIter      *where);
-
-void                     gtk_source_buffer_move_marker          (GtkSourceBuffer        *buffer,
-								 GtkSourceMarker        *marker,
-								 const GtkTextIter      *where);
-void                     gtk_source_buffer_delete_marker        (GtkSourceBuffer        *buffer,
-								 GtkSourceMarker        *marker);
-GtkSourceMarker         *gtk_source_buffer_get_marker           (GtkSourceBuffer        *buffer,
-								 const gchar            *name);
-GSList                  *gtk_source_buffer_get_markers_in_region
-                                                                (GtkSourceBuffer        *buffer,
-								 const GtkTextIter      *begin,
-								 const GtkTextIter      *end);
-GtkSourceMarker         *gtk_source_buffer_get_first_marker     (GtkSourceBuffer        *buffer);
-GtkSourceMarker         *gtk_source_buffer_get_last_marker      (GtkSourceBuffer        *buffer);
-void                     gtk_source_buffer_get_iter_at_marker   (GtkSourceBuffer        *buffer,
+gboolean		 gtk_source_buffer_forward_iter_to_mark	(GtkSourceBuffer        *buffer,
 								 GtkTextIter            *iter,
-								 GtkSourceMarker        *marker);
-GtkSourceMarker         *gtk_source_buffer_get_next_marker      (GtkSourceBuffer        *buffer,
-								 GtkTextIter            *iter);
-GtkSourceMarker         *gtk_source_buffer_get_prev_marker      (GtkSourceBuffer        *buffer,
-								 GtkTextIter            *iter);
-
+								 const gchar            *category);
+gboolean		 gtk_source_buffer_backward_iter_to_mark(GtkSourceBuffer        *buffer,
+								 GtkTextIter            *iter,
+								 const gchar            *category);
+GSList			*gtk_source_buffer_get_marks_at_iter    (GtkSourceBuffer        *buffer,
+								 GtkTextIter            *iter,
+								 const gchar            *category);
+GSList			*gtk_source_buffer_get_marks_at_line	(GtkSourceBuffer        *buffer,
+								 gint 			 line,
+								 const gchar		*category);
+void			 gtk_source_buffer_remove_marks		(GtkSourceBuffer        *buffer,
+								 GtkTextIter            *start,
+								 GtkTextIter            *end,
+								 const gchar            *category);
 /* private */
 void			 _gtk_source_buffer_update_highlight	(GtkSourceBuffer        *buffer,
 								 const GtkTextIter      *start,
 								 const GtkTextIter      *end,
 								 gboolean                synchronous);
 
+GtkSourceMark		*_gtk_source_buffer_mark_next		(GtkSourceBuffer        *buffer,
+								 GtkSourceMark          *mark, 
+								 const gchar            *category);
+GtkSourceMark		*_gtk_source_buffer_mark_prev		(GtkSourceBuffer        *buffer,
+								 GtkSourceMark          *mark, 
+								 const gchar            *category);
 G_END_DECLS
 
 #endif /* __GTK_SOURCE_BUFFER_H__ */
